@@ -300,27 +300,33 @@ int am_check_permissions(request_rec *r, am_cache_entry_t *session)
          */
         for (j = 0; (j < session->size) && !match; j++) {
             const char *varname = NULL;
+            am_envattr_conf_t *envattr_conf = NULL;
 
             /*
              * if MAP flag is set, check for remapped 
              * attribute name with mellonSetEnv
              */
-            if (ce->flags & AM_COND_FLAG_MAP)
-                varname = apr_hash_get(dir_cfg->envattr, 
-                                       session->env[j].varname, 
-                                       APR_HASH_KEY_STRING);
+            if (ce->flags & AM_COND_FLAG_MAP) {
+                envattr_conf =  (am_envattr_conf_t *)apr_hash_get(dir_cfg->envattr, 
+                                         am_cache_entry_get_string(session,&session->env[j].varname),
+                                         APR_HASH_KEY_STRING);
+                                                    
+                if (envattr_conf != NULL)
+                    varname = envattr_conf->name;
+            }
 
             /*
              * Otherwise or if not found, use the attribute name
              * sent by the IdP.
              */
             if (varname == NULL)
-                varname = session->env[j].varname;
+                varname = am_cache_entry_get_string(session,
+                                                    &session->env[j].varname);
                       
             if (strcmp(varname, ce->varname) != 0)
                     continue;
 
-            value = session->env[j].value;
+            value = am_cache_entry_get_string(session, &session->env[j].value);
 
             /*
              * Substiture backrefs if available
@@ -391,9 +397,7 @@ int am_check_permissions(request_rec *r, am_cache_entry_t *session)
     return OK;
 }
 
-
-/* This function disables caching of the response to this request. It does
- * this by setting the Pragme: no-cache and Cache-Control: no-cache headers.
+/* This function sets default Cache-Control headers.
  *
  * Parameters:
  *  request_rec *r       The request we are handling.
@@ -401,31 +405,21 @@ int am_check_permissions(request_rec *r, am_cache_entry_t *session)
  * Returns:
  *  Nothing.
  */
-void am_set_nocache(request_rec *r)
+void am_set_cache_control_headers(request_rec *r)
 {
-     const char *user_agent;
-
-    /* Setting the headers inn err_headers_out ensures that they will be
+    /* Send Cache-Control header to ensure that:
+     * - no proxy in the path caches content inside this location (private),
+     * - user agent have to revalidate content on server (must-revalidate).
+     *
+     * But never prohibit specifically any user agent to cache or store content
+     *
+     * Setting the headers in err_headers_out ensures that they will be
      * sent for all responses.
      */
     apr_table_setn(r->err_headers_out,
-		   "Expires", "Thu, 01 Jan 1970 00:00:00 GMT");
-    apr_table_setn(r->err_headers_out,
-		   "Cache-Control", "private, must-revalidate");
-
-    /* 
-     * Never use Cache-Control: no-cache for IE
-     */
-    user_agent = apr_table_get(r->headers_in, "User-Agent");
-    if ((user_agent == NULL) ||
-         (strstr(user_agent, "compatible; MSIE ") == NULL) ||
-         (strstr(user_agent, "Opera") != NULL)) {
-        apr_table_addn(r->err_headers_out,
-		       "Cache-Control", "no-cache, no-store");
-    }
+                   "Cache-Control", "private, must-revalidate");
 }
 
-
 /* This function reads the post data for a request.
  *
  * The data is stored in a buffer allocated from the request pool.
@@ -625,6 +619,10 @@ int am_urldecode(char *data)
     char *op;
     int c1, c2;
 
+    if (data == NULL) {
+        return HTTP_BAD_REQUEST;
+    }
+
     ip = data;
     op = data;
     while (*ip) {
@@ -795,19 +793,14 @@ int am_check_url(request_rec *r, const char *url)
 int am_generate_random_bytes(request_rec *r, void *dest, apr_size_t count)
 {
     int rc;
-    rc = RAND_pseudo_bytes((unsigned char *)dest, (int)count);
-    if(rc == -1) {
+    rc = RAND_bytes((unsigned char *)dest, (int)count);
+    if(rc != 1) {
         ap_log_rerror(APLOG_MARK, APLOG_ERR, 0, r,
                       "Error generating random data: %lu",
                       ERR_get_error());
         return HTTP_INTERNAL_SERVER_ERROR;
     }
 
-    if(rc == 0) {
-        ap_log_rerror(APLOG_MARK, APLOG_WARNING, 0, r,
-                      "Random data is not cryptographically strong.");
-    }
-
     return OK;
 }
 
@@ -1355,8 +1348,8 @@ const char *am_get_mime_header(request_rec *r, const char *m, const char *h)
 
         if (((value = am_xstrtok(r, line, ":", &l2)) != NULL) &&
             (strcasecmp(value, h) == 0)) {
-            value =  am_xstrtok(r, NULL, ":", &l2);
-            am_strip_blank(&value);
+            if ((value = am_xstrtok(r, NULL, ":", &l2)) != NULL)
+                am_strip_blank(&value);
             return value;
         }
    }
@@ -1407,9 +1400,9 @@ const char *am_get_header_attr(request_rec *r, const char *h,
 
         attr_name = am_xstrtok(r, attr, "=", &l2); 
         if ((attr_name != NULL) && (strcasecmp(attr_name, a) == 0)) {
-        	attr_value = am_xstrtok(r, NULL, "=", &l2);
+            if ((attr_value = am_xstrtok(r, NULL, "=", &l2)) != NULL)
                 am_strip_blank(&attr_value);
-        	break;
+            break;
         }
     }
   
